# -*- coding: utf8 -*-
from __future__ import unicode_literals

import time
import datetime

from driverng.utils import FS
from driverng.protocol.mllp import MLLPProtocol


class MasslynxProtocol(MLLPProtocol):

    def mllp_readline(self):
        return self.readline(break_func=lambda d: d and d[-2:] == b"%c\r" % (
            FS,))

    def get_ack_message(self, payload, date=None, control_id=None):
        splitted = payload.splitlines()[0].split(b"|")
        date = date or datetime.datetime.now()
        control_id = b"%d" % int(time.time() * 1000) if not control_id else control_id
        context = {
            b"date": date.strftime("%Y%m%d%H%M%S").encode(),
            b"control_id": control_id,
            b"sender_id": splitted[4],
            b"receiver_id": splitted[2],
            b"ack_id": splitted[9],
            b"message_type": self.ACK_MESSAGE_TYPE[self.is_automate],
        }
        return self.HL7_ACK % context

    def recv_trames(self):
        data = self.mllp_readline()
        messages = []
        acks = []
        for d in data.split(b"\x1c\x0d"):  # FS CR
            if d:
                payload = self.get_payload(d)
                self.logger.info("recv trames: %s ", payload)
                ack = self.get_ack_message(payload)
                if ack:
                    acks.append(ack)
                messages.append(payload)
        for ack in acks:
            self.write(self.build_trames(ack))
        return messages

    @staticmethod
    def get_payload(data):
        return data[1:]
